Package: rtdists
Type: Package
Title: Response Time Distributions
Depends: R (>= 3.0.0)
Suggests: testthat, glba, knitr, rmarkdown, dplyr, tidyr, purrr,
        lattice, latticeExtra, binom, RWiener
Imports: evd, msm, gsl, stats, Rcpp
LinkingTo: Rcpp
Description: Provides response time distributions (density/PDF,
       distribution function/CDF, quantile function, and random
       generation): (a) Ratcliff diffusion model (Ratcliff &
       McKoon, 2008, <doi:10.1162/neco.2008.12-06-420>) based on C
       code by Andreas and Jochen Voss and (b) linear ballistic
       accumulator (LBA; Brown & Heathcote, 2008,
       <doi:10.1016/j.cogpsych.2007.12.002>) with different
       distributions underlying the drift rate.
URL: https://github.com/rtdists/rtdists/
License: GPL (>= 3)
BugReports: https://github.com/rtdists/rtdists/issues
VignetteBuilder: knitr
Authors@R: c( person(given="Henrik", family="Singmann",
       email="singmann@gmail.com", role=c("aut", "cre"),
       comment=c(ORCID="0000-0002-4842-3657")),
       person(given="Scott", family="Brown", role=c("aut")),
       person(given="Matthew", family="Gretton", role=c("aut")),
       person(given="Andrew", family="Heathcote", role=c("aut")),
       person(given="Andreas", family="Voss", role=c("ctb")),
       person(given="Jochen", family="Voss", role=c("ctb")),
       person(given="Andrew", family="Terry", role=c("ctb")) )
LazyData: true
Version: 0.11-6
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-06-10 06:39:55 UTC; singmann
Author: Henrik Singmann [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4842-3657>),
  Scott Brown [aut],
  Matthew Gretton [aut],
  Andrew Heathcote [aut],
  Andreas Voss [ctb],
  Jochen Voss [ctb],
  Andrew Terry [ctb]
Maintainer: Henrik Singmann <singmann@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-10 09:00:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-06-10 09:35:03 UTC; unix
Archs: rtdists.so.dSYM
