# Generated by roxygen2: do not edit by hand

export(dLBA)
export(ddiffusion)
export(dlba_frechet)
export(dlba_gamma)
export(dlba_lnorm)
export(dlba_norm)
export(drd)
export(n1CDF)
export(n1PDF)
export(pLBA)
export(pdiffusion)
export(plba_frechet)
export(plba_gamma)
export(plba_lnorm)
export(plba_norm)
export(prd)
export(qLBA)
export(qdiffusion)
export(rLBA)
export(rdiffusion)
export(rlba_frechet)
export(rlba_gamma)
export(rlba_lnorm)
export(rlba_norm)
export(rrd)
importFrom(Rcpp,evalCpp)
importFrom(evd,dfrechet)
importFrom(evd,pfrechet)
importFrom(evd,rfrechet)
importFrom(gsl,gamma_inc)
importFrom(msm,rtnorm)
importFrom(stats,dgamma)
importFrom(stats,dlnorm)
importFrom(stats,dnorm)
importFrom(stats,integrate)
importFrom(stats,optimize)
importFrom(stats,pgamma)
importFrom(stats,plnorm)
importFrom(stats,pnorm)
importFrom(stats,rgamma)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,uniroot)
useDynLib(rtdists, .registration = TRUE)
