#' Response Time Distributions.
#'
#' \tabular{ll}{
#' Package: \tab rtdists\cr
#' Type: \tab Package\cr
#' Version: \tab 0.8-3\cr
#' Date: \tab 2018-06-23\cr
#' Depends: \tab R (>= 3.0.0)\cr
#' License: \tab GPL (>=3)\cr
#' URL: \tab https://github.com/rtdists/rtdists/\cr
#' }
#'
#' Provides response time distributions (density/PDF, distribution function/CDF, quantile
#' function, and random generation): (a) Ratcliff diffusion model (Ratcliff & McKoon, 2008,
#' <doi:10.1162/neco.2008.12-06-420>) based on C code by Andreas and Jochen Voss and (b) linear
#' ballistic accumulator (LBA; Brown & Heathcote, 2008, <doi:10.1016/j.cogpsych.2007.12.002>)
#' with different distributions underlying the drift rate.
#'
#' @aliases rtdists-package
#' @name rtdists-package
#' @docType package
#' @title The rtdists Package
#' @author Henrik Singmann, Scott Brown, Matthew Gretton, Andrew Heathcote, with contributions from Andreas Voss, Jochen Voss, Andrew Terry
#' @keywords package
NULL
