# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

d_fastdm <- function(rts, params, precision = 3, boundary = 2L, stop_on_error = TRUE) {
    .Call(`_rtdists_d_fastdm`, rts, params, precision, boundary, stop_on_error)
}

p_fastdm <- function(rts, params, precision = 3, boundary = 2L, stop_on_error = TRUE) {
    .Call(`_rtdists_p_fastdm`, rts, params, precision, boundary, stop_on_error)
}

p_precise_fastdm <- function(rts, params, precision = 3, boundary = 2L, stop_on_error = TRUE) {
    .Call(`_rtdists_p_precise_fastdm`, rts, params, precision, boundary, stop_on_error)
}

r_fastdm <- function(num_values, params, precision = 3, stop_on_error = TRUE) {
    .Call(`_rtdists_r_fastdm`, num_values, params, precision, stop_on_error)
}

