% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rtdists-package.R
\docType{package}
\name{rtdists-package}
\alias{rtdists}
\alias{rtdists-package}
\title{The rtdists Package}
\description{
Response Time Distributions.
}
\details{
\tabular{ll}{
Package: \tab rtdists\cr
Type: \tab Package\cr
Version: \tab 0.8-3\cr
Date: \tab 2018-06-23\cr
Depends: \tab R (>= 3.0.0)\cr
License: \tab GPL (>=3)\cr
URL: \tab https://github.com/rtdists/rtdists/\cr
}

Provides response time distributions (density/PDF, distribution function/CDF, quantile
function, and random generation): (a) Ratcliff diffusion model (Ratcliff & McKoon, 2008,
<doi:10.1162/neco.2008.12-06-420>) based on C code by Andreas and Jochen Voss and (b) linear
ballistic accumulator (LBA; Brown & Heathcote, 2008, <doi:10.1016/j.cogpsych.2007.12.002>)
with different distributions underlying the drift rate.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/rtdists/rtdists/}
  \item Report bugs at \url{https://github.com/rtdists/rtdists/issues}
}

}
\author{
Henrik Singmann, Scott Brown, Matthew Gretton, Andrew Heathcote, with contributions from Andreas Voss, Jochen Voss, Andrew Terry
}
\keyword{package}
