% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{drd}
\alias{drd}
\alias{rtdists-deprecated}
\alias{prd}
\alias{rrd}
\title{Deprecated functions}
\usage{
drd(...)

prd(...)

rrd(...)
}
\arguments{
\item{...}{arguments passed from the old functions to the new functions}
}
\description{
These functions have been renamed and deprecated in \pkg{rtdists}:
\code{drd()} (use \code{\link{ddiffusion}()}), 
\code{prd()} (use \code{\link{pdiffusion}()}),
\code{rrd()} (use \code{\link{rdiffusion}()}).
}
\keyword{internal}
